@@ -2,6 +2,13 @@
 
 #include "config.h"
 
+#ifdef __NetBSD__
+#include <termios.h>
+#define termio termios
+typedef struct termios SGTTY;
+#define TCSETAW TIOCSETAW
+#define TCGETA TIOCGETA
+#endif
 #ifdef CURSES
 
 #ifdef USG
@@ -44,6 +51,12 @@
 #include <setjmp.h>
 #include "strings.h"
 
+#ifdef MSGS_DYNAMIC
+#if defined(sparc) || defined(__sparc__)
+#include <alloca.h>
+#endif
+#endif
+
 extern char
     *malloc(),		/* allocate memory */
     *calloc(),		/* allocate and clear memory */
@@ -166,16 +179,22 @@ extern int bell();
 #endif /* TIOCSETN */
 
 /* for system-V machines that run termio */
-#if defined(SYSV) && defined(USG) || defined(AIX)
+#if defined(SYSV) && defined(USG) || defined(AIX) || defined(__NetBSD__)
 #ifdef crmode
 #undef crmode
 #undef nocrmode
 #endif /* nocrmode */
 
 unsigned char vmin, vtime;
-#define sg_erase  c_cc[2]
+#ifndef VERASE
+#define VERASE 2
+#endif
+#ifndef VKILL
+#define VKILL 3
+#endif
+#define sg_erase  c_cc[VERASE]
 #define sg_flags  c_lflag
-#define sg_kill   c_cc[3]
+#define sg_kill   c_cc[VKILL]
 #define sg_ospeed c_cflag
 #define gtty(fd, SGTTYbuf)	ioctl(fd, TCGETA, SGTTYbuf)
 #undef stty
@@ -194,7 +213,7 @@ unsigned char vmin, vtime;
 
 /* If curses isn't defined, declare our 'tty' and macros for echo/cbreak */
 #ifndef CURSES
-typedef struct termio SGTTY;
+typedef struct termios SGTTY;
 #define echom()    echon()
 #define noechom()  echoff()
 #define crmode()   cbrkon()
@@ -211,7 +230,7 @@ typedef struct termio SGTTY;
 #endif /* CURSES */
 #endif /* SYSV && USG || AIX */
 
-#if !defined(USG) && !defined(AIX)
+#if !defined(USG) && !defined(AIX) && !defined(__NetBSD__)
 #ifndef CURSES
 /* if curses is not defined, simulate the same tty based macros */
 typedef struct sgttyb SGTTY;
@@ -437,7 +456,12 @@ u_long   glob_flags;	/* global boolean flags thruout the whole program */
 /* It is possible to reset MAX_PRIORITY to as high as 9 */
 #define MAX_PRIORITY	5
 
+#ifndef MSGS_DYNAMIC
 #define	MAXMSGS_BITS	MAXMSGS/sizeof(char)	/* number of bits for bitmap */
+#else
+#define MSGS_BITS(a)	(((a)+7)/8)
+#define MSGS_INCR	500
+#endif
 
 struct msg {
     u_long m_flags;
@@ -446,7 +470,12 @@ struct msg {
     int    m_lines;	/* number of lines in msg */
     char   *m_date_recv;/* Date user received msg (see dates.c for fmt) */
     char   *m_date_sent;/* Date author sent msg (see dates.c for fmt) */
-} msg[MAXMSGS];
+} 
+#ifdef MSGS_DYNAMIC
+    *msg;
+#else
+    msg[MAXMSGS];
+#endif
 
 struct options {
     char *option;
@@ -477,7 +506,9 @@ FILE
     *popen();		/* this should be in stdio.h */
 
 extern char
+#ifndef __NetBSD__
     *sys_errlist[],    /* system's list of global error messages */
+#endif
     **environ;		/* user's environment variables */
 
 extern int errno;	/* global system error number */
@@ -486,7 +517,11 @@ jmp_buf jmpbuf;		/* longjmp to jmpbuf on sigs (not in tool) */
 char
     debug,		/* debug causes various print statements in code */
     tempfile[MAXPATHLEN],	/* path to filename of temporary file */
+#ifdef MSGS_DYNAMIC
+    *msg_list,		/* boolean storage */
+#else
     msg_list[MAXMSGS_BITS],	/* MAXMSGS bits of boolean storage */
+#endif
     **alternates,	/* alternates list --see alts() */
     *cmd_help,		/* filename of location for "command -?" commands. */
     *login,		/* login name of user */
@@ -540,6 +575,9 @@ char
 int
     last_msg_cnt,	/* when checking for new mail, save the last msg_cnt */
     msg_cnt,		/* total number of messages */
+#ifdef MSGS_DYNAMIC
+    msg_max,		/* current maximum allocation limit */
+#endif
     crt,		/* min number of lines msg contains to invoke pager */
     current_msg,	/* the current message we're dealing with */
     exec_pid,		/* pid of a command that has been "exec"ed */
@@ -589,9 +627,9 @@ void
 #ifdef TIOCGLTC
 struct ltchars ltchars;			/* tty character settings */
 #endif /* TIOCGLTC */
-#if defined(BSD) && !defined(AIX) /* (TIOCGETC) */
+#if defined(BSD) && !defined(AIX) && defined(TIOCGETC)
 struct tchars  tchars;			/* more tty character settings */
-#endif /* BSD && !AIX (TIOCGETC) */
+#endif /* BSD && !AIX && TIOCGETC */
 
 #ifdef CURSES
 
